<CsoundSynthesizer>
<CsInstruments>
/**********************************************************************

	MacCsound Tutorial #3
	

	Csound Language Basics.


	Csound is a kind of scripting language to define synthesis and manipulation
	of digital audio.  You define your scripts in an "orchestra" [orc] part
	and a "score" [sco] part.

	The orc is where you define your instruments [patches].  
	The sco is where you allocate instances of these instruments.  Later we will
	find out how to allocate instrument instances in real-time from the i/o panel
	and from another instrument!
	
	Let's now look at the details of a simple sine wave instrument:
*/



; all instrument definitions must first begin with an 'instr' call
; you can name your instrument with a number or name.  here we
; just name it by a number (1).
instr 1	

/* 	this next non-commented line below creates an oscillator, by using
	the 'oscili' opcode.  to view its manual entry, select the oscili 
	opcode and press cmd-'.  you will see the syntax of the opcode is:
	
		ar oscili xamp, xcps, ifn [, iphs]
	
	remember that all input paramaters are to the RIGHT of the opcode and all
	output pramaters are to the LEFT of the opcode -- so what we are seeing
	is that the 'oscili' opcode outputs one audio signal "ar", 
	and its input parameters are amplitude, cycles per second, 
	wave table to oscillate, and [an optional] phase value.

	so we define our oscillator below with an amplitude of 10000, 
	a pitch of 440Hz, using wave table 1, and no phase value.
	the output is put into the variable 'asig'. [ we will see in the
	'sco' panel how the wave table is defined ]
*/
	asig oscili 10000, 440, 1

	; we now need to send our signal to the speakers.  this is done with the
	; 'out' opcode.  for its input we give it the 'asig' variable, which 
	; contains the output of the oscil.

	out asig	; send the signal to the output

; Finally, we must end the instrument definition with an 'endin' opcode.
; REMEMBER THIS - it is a common mistake for beginners to forget the 'endin' 
endin			




; now go to the 'sco' to view the rest of our code........................

</CsInstruments>
<CsScore>

; we first must define a waveform for the 'oscili' opcode to use
; the wave is defined in a function table.  a function table is indeed
; a table [a list] of values.  you always define a table in more or less
; the same way, but there are slightly different 'modes' how to do this.

; here, we define a sine wave:

f1 0 512 10 1

; "f1" means we are defining function table number 1
; "0" means allocate the table at time 0
; "512" is the size of the table [sizes must always be a power of 2]
; "10" is the "generation routine" [gen 10 is used to define sine waves in a harmonic series]
; "1" (in gen10) means the first partial with strength 1 (in essence a sine wave)


; finally, we must define an instance of our instrument, 
; done like so with an "i-statement":

i1 0 5

; "i1" means a new allocation of instrument 1
; "0" means start at time zero
; "5" means play for a duration of 5 seconds



;**********************************************************************
; NOW hear the sound by pressing the (green triangle) "play" button!
;**********************************************************************




; Before going on to the next tutorial,
; + try different start and duration values for the i-statement above
; + go back to the 'orc' panel and try changing the 
; 	amp and cps input parameters to the oscili opcode
; + try adding an additional i-statements 





;**********************************************************************
;	At this point, you might want to read Dr. Richard Boulanger's
;	introduction to the Csound language, found here:
;
;		http://www.csounds.com/toots/TootIntro.html
;**********************************************************************
</CsScore>

</CsoundSynthesizer>
<MacOptions>
Version: 3
Render: Real
Ask: Yes
Functions: Window
Listing: ioObject
WindowBounds: 107 44 965 768
CurrentView: orc
IOViewEdit: On
Options: -b512 -A -s -m7 -R 
</MacOptions>
<MacGUI>
ioView nobackground {65535, 65535, 65535}
ioListing {10, 10} {400, 500}
</MacGUI>
