<CsoundSynthesizer>
<CsInstruments>
/**********************************************************************

	MacCsound Tutorial #7

	
	Rates, etc.

	
	You may have noticed in the previous tutorials that many
	variables started with an 'a' or 'k'.  This is not a mere
	coincidence.  Csound's opcodes and variables run at 
	different 'rates', and the rate of a variable is 
	specified by its first letter.

	There are currently 4 variable rates in Csound:

		i - initialization rate, usually used for constant values
		k - control rate (kr), usually used for control signals
		a - audio "sample" rate (sr), usually used for audio signals
		f - fft rate [used only for streaming pvoc opcodes]


	You specify the sample rate and control rate in the "header" 
	portion of the orc, before any instrument declarations.  Control
	rate can be specified directly, or with the 'ksmps' parameter,
	or both. [ksmps = sr/kr, and must be a whole integer]  In addition,
	you can specify the number of audio channels.  If not specified, 
	sr defaults to 44100, kr = 4410, and nchnls = 1.
*/


	sr = 44100 	; samples per second - can be fractional, but usually is not
	kr = 4410	; control values per second - can be fractional, but usually is not
	ksmps = 10	; sr/kr - must be a whole integer
	nchnls = 2	; number of audio channels in/out


; Here is a csound instrument that shows variable use in the 3 common rates
instr 1

	; i-rate stuff
	itable = 1
	iRightTable = 2
	iCps = 300

	; k-rate stuff		[ oscillate a 1/2 sine @ 2hz for panning ]
	kLeftAmp 	oscili 	0dbfs * .5, 2, 2		; (0dbfs = maxvolume)
	kRightAmp  	oscili 	0dbfs * .5, 2, 2, .5	; start it the middle of the table
	
	; a-rate stuff
	aAudioLeftChannel 	oscili	kLeftAmp, iCps, itable	
	aAudioRightChannel 	oscili	kRightAmp, iCps, itable
	
	outs	aAudioLeftChannel, aAudioRightChannel ; sound stero out 

endin




/*
	Why use different rates?


	Digital audio is very processor intensive, and for many "control"
	kind of signals, you do not need them updating tens of thousands 
	of times a second.  So the "control" rate in Csound is used to 
	direct the processing power to where it is needed: in generating
	audio signals.

	Most of the time, you only need to think as far as using
	'i-rate' variables for constant values, 'a-rate' variables 
	for audio signals and 'k-rate' variables for control signals.  
	However, sometimes you run into situations where you need to 
	understand how Csound's rates actually work.

	When an instrument is instantiated by an "i" statement in
	the score, an initialization pass is first performed.  
	Flowing from top to bottom, all opcodes are allocated 
	and given a chance to perform any "i-rate" calculation.  
	In addition, all "i-rate" variables are assigned to 
	their values, which do not change for the entire duration 
	of the instrument's note.

	Then, as the instrument is "performed", opcodes are executed
	in a continuing cycle, in order from top to bottom and then 
	repeating the cycle back at the top.   During each	execution 
	pass, both 'k-rate' and 'a-rate' variables are processed.  A 
	'k-rate' variable represents one value, while an 'a-rate' 
	variable represents many "samples".  The number of values is
	actually equal to ksmps. [that is why ksmps must be an integer, 
	it would be pretty hard to generate a fraction of a sample!]
*/


</CsInstruments>
<CsScore>
f1 0 512 10 1 1 1 1 1 1 ; buzz wave
f2 0 512 9 .5 1 0 ; 1/2 sine wave for pan function

i1 0 10
</CsScore>

</CsoundSynthesizer>
<MacOptions>
Version: 3
Render: Real
Ask: Yes
Functions: Window
Listing: ioObject
WindowBounds: 30 44 876 687
CurrentView: orc
IOViewEdit: On
Options: -b512 -A -s -m7 -R 
</MacOptions>
<MacGUI>
ioView nobackground {65535, 65535, 65535}
ioListing {10, 10} {400, 500}
</MacGUI>
